@import "common.css";

/* color style */
.code q::before , .code q::after { content:none; }
.code dfn { font-style: normal }

.code b, .code em { color: #808000; font-style:normal; font-weight: normal}
.code i, .code q, .code kbd, .code kdb, .code u>a, .code .string { color: #008000 ; font-style:normal; font-weight: normal}
.code i.doc { color: #000080; }
.code i .command { color: #00f }
.code i a { color: #037; }
.code var, .code u, .code .macro { color: #000080;  text-decoration: none; font-style:normal }

.code .type, .code .typedef, .code .namespace, .code .enum  { color: #800080}
.code .field, .code .member, .code .lbl { color: #860d0d }
.code .virtual { font-style: italic }
.code .local { color: #092e64 }
.code .lbl { color: #800000 }
.code .global { color: #ce5c00 } /* FIXME */
.code .fn { color: #00677c }


.highlight { background-color: #eee; border-radius:3px; border: 1px solid #B4B4B4; margin:-1px}
th.highlight { font-weight: bold; }
th.highlight+td { background-color: #eee; }

.sideBox { border: 1px solid #ddd; background-color: #eee; }

