@import "common.css";

/* color style */
/*.comment, */  .code i { color: #888; font-style: oblique }
                .code i.doc { color: #00f; font-style: oblique }
                .code i .command { color: #CA60CA; font-weight: bold; font-style: normal;}
                .code i .arg { color: #0095FF; font-weight: bold; font-style: normal;}
                .code i .tag { color: #000; font-weight: bold; font-style: normal;}
                .code i .verb { color: #888; }
                .code i .command  + .verb, .code i .command  + a .verb { color: #f00; font-style:normal }
                .code i a { color: #037; }
/*.keyword, */  .code b { font-weight: bold; }
/*.keytype, */  .code em { color: #0057AE; font-style:normal }
.code .string,  .code q { color: #BF0303; font-style: normal }
.code q::before , .code q::after { content:none; }
/*.directive,*/ .code u { color: #063; text-decoration:none; }
/*.num,      */ .code var { color: #B08000; font-style: normal; }
/*.char,     */ .code kdb, .code kbd { color: #FF80E0 }

/* code a,  code dfn  */
.code .ref { color: #208 }
.code .type , #tooltip .type { color: #061 }
.code .typedef { color: #35938D }
.code .member { color: #B08000 }
.code .call { color: #207 }
.code .namespace, .code .enum { color: #862a38 }
.code .decl { color: #205; font-weight:bold; }
.code .macro { background-color:#efe; color: inherit; }
.code .macro:hover { background-color:#BEFF9B; }
.code .macro.highlight { background-color:#BEFF9B !important; }

.code .col0 {color:#2400FF} .code .col1 {color:#9C00FF} .code .col2 {color:#D1008A} .code .col3 {color:#FF0000} .code .col4 {color:#FF4F00}
.code .col5 {color:#F69D00} .code .col6 {color:#72D200} .code .col7 {color:#00F718} .code .col8 {color:#00BBCD} .code .col9 {color:#0053FF}

.code dfn {  font-style:normal }
.code .virtual { font-style: italic; }

.highlight { background-color: #FBFA96 !important; color: black !important;}
th.highlight { font-weight: bold; }
th.highlight+td { background-color: #f8f7f6; }

.sideBox { border: 1px solid #ddd; background-color: #eee; }
