@import "jquery/jquery-ui.css";

body { color:#000000; background-color:#ffffff; padding:0; margin: 0;  font-family: Helvetica,sans-serif; }
a { color: #037; border: none; text-decoration: none }
a:visited { color: #737; }
a:hover{  text-decoration: underline }
#footer { font-size: smaller; margin:1ex; color: #333; text-align: right }
#footer img { vertical-align: middle; }
#whatisit { max-width: 30em; margin: auto; padding:1ex; }
img { border: none; }
input#searchline { margin: 1ex;  width: 30em; max-width: 50%; }
@media only screen and (max-width: 30em) {
    input#searchline {max-width: 90%; }
}
p {  margin:0 }

/* 1.9 logo, we keep this for compatibility for a common data/ directory */
h1 { float: right; padding: 0px; margin:1px;
    background-image: url(woboq-48.png); background-repeat: no-repeat;
    height:48px; width: 126px; overflow:hidden
}
h1 a { visibility:hidden }

h2 { font-size:medium; font-weight: normal;
        padding:0; margin: 0 1ex 2ex 1ex;   }
/* end compatibility */
div#toprightlogo {
    float: right; padding: 0px; margin:1px;
    background-image: url(woboq-48.png); background-repeat: no-repeat;
    height:48px; width: 126px; overflow:hidden
}
@media only screen and (max-width: 30em) {
    div#toprightlogo {display:block;float:none; margin:1em auto;  padding: 0}
}
/* new h1 breadcrumb: */
h1#title { float: none; background-image: none; height:auto;width:auto; font-size:1em;
font-weight:normal;  margin-left:1ex;}
h1#title a { visibility: visible }


a { text-decoration:none}

div#header { width:100%; background-color: #f5f5f5; min-height:5.2em; display:block;}
div#header input {-webkit-appearance:searchfield;-moz-appearance:searchfield;appearance:searchfield;}

hr { margin: 0; border-collapse:collapse;background-color: #e5e5e5; height:1px;border:none;  }

table#tree { padding: 1ex 1ex; }
table#tree { text-align:left; margin: auto}


table#tree table { padding:0; padding-left: 4ex; margin:0; border:0; }
table#tree td { white-space: pre; font-family:monospace; margin:0; padding:0; border: 0 }
table#tree tr { margin:0; border:0; padding:0 }

table#tree tr:nth-child(even) { background-color: #fafafa }
table#tree tr table { background-color: white }

/*
table#tree tr:hover  { background-color: #eee }
table#tree tr:hover table { background-color: white }*/

td  a { padding: 1px; }
td.folder > a { padding-left: 22px; background: url("folder.png") no-repeat left }
td.file > a { padding-left: 22px; background: url("txt.png") no-repeat left }
td.parent > a { padding-left: 22px; background: url("back.png") no-repeat left }

td a.opener { color:inherit; background: none; padding:0 }

/* class list */
td span.meta { white-space:pre-line;display:inline ; float:right;}
td span.meta a { }
td span.meta ul { margin: 0; padding:0; display:inline ;}
td span.meta ul li { list-style-type: none; padding:0; margin:0;padding-left:1ex; }

/* Hide [+], make more list-ish */
@media only screen and (max-width: 30em) {
    table#tree td { white-space: pre-line; }
    td a.opener { display : none; }
    td span.meta { display: block; float: none;}
    td > a { margin-bottom: 1em !important; display:block;}
    td span.meta ul li { margin-bottom: 1em ; }
}
